# -*- coding: utf-8 -*-
# Generated by Django 1.11.5 on 2018-04-24 16:12
#
# This migration adds a new "emfollow" user which will be used for
# EM follow-up activities.  The old EM follow-up accounts will be deactivated
# and their X509Certs will be deleted.

from __future__ import unicode_literals

from django.db import migrations

# Previously, this was taken from settings.LVC_GROUP, but that value has
# changed. So we have to hard-code it for past migrations.
LVC_GROUP = 'Communities:LSCVirgoLIGOGroupMembers'


OLD_ACCOUNTS = {
    'gdb-processor': {
        'email': 'gdb_processor@gravity.phys.uwm.edu',
        'last_name': 'GDB Processor',
        'certs': [
            '/DC=org/DC=ligo/O=LIGO/OU=Services/CN=gdb_processor/pcdev2.phys.uwm.edu',
            '/DC=org/DC=ligo/O=LIGO/OU=Services/CN=gdb-processor/marlin.phys.uwm.edu',
        ],
    },
    'gracedb.processor': {
        'email': 'alexander.urban@ligo.org',
        'last_name': 'GraceDB Processor',
        'certs': [
            '/DC=org/DC=ligo/O=LIGO/OU=Services/CN=gracedb.processor/ldas-grid.ligo.caltech.edu',
            '/DC=org/DC=ligo/O=LIGO/OU=Services/CN=gracedb.processor/emfollow.ligo.caltech.edu',
        ],
    },
}

NEW_ACCOUNT = {
    'username': 'emfollow',
    'last_name': 'LIGO/Virgo EM Follow-Up',
    'email': 'leo.singer@ligo.org',
}

NEW_CERTS = [
    '/DC=org/DC=ligo/O=LIGO/OU=Services/CN=emfollow/emfollow.ligo.caltech.edu'
]

def deactivate_old_and_add_new_accounts(apps, schema_editor):
    LocalUser = apps.get_model('ligoauth', 'LocalUser')
    X509Cert = apps.get_model('ligoauth', 'X509Cert')
    Group = apps.get_model('auth', 'Group')

    # Mark old users as inactive and delete their certificates
    for username in OLD_ACCOUNTS:
        user = LocalUser.objects.get(username=username)
        for subject in OLD_ACCOUNTS[username]['certs']:
            cert = user.x509cert_set.get(subject=subject)
            cert.delete()
        user.is_active = False
        user.save()

    # Create new EM follow-up user
    new_user = LocalUser.objects.create(**NEW_ACCOUNT)

    # Create and add certificate for new account
    for subject in NEW_CERTS:
        new_user.x509cert_set.create(subject=subject)

    # Add user to LVC group
    group = Group.objects.get(name=LVC_GROUP)
    group.user_set.add(new_user)

def activate_old_and_remove_new_accounts(apps, schema_editor):
    LocalUser = apps.get_model('ligoauth', 'LocalUser')
    X509Cert = apps.get_model('ligoauth', 'X509Cert')

    # Activate old accounts and add their X509 certificates
    for username in OLD_ACCOUNTS:
        user = LocalUser.objects.get(username=username)
        for subject in OLD_ACCOUNTS[username]['certs']:
            cert = user.x509cert_set.create(subject=subject)
        user.is_active = True
        user.save()

    # Delete new EM follow-up user and certs
    new_user = LocalUser.objects.get(username=NEW_ACCOUNT['username'])
    for subject in NEW_CERTS:
        cert = new_user.x509cert_set.get(subject=subject)
        cert.delete()
    new_user.delete()

    # Create and add certificate for new account
class Migration(migrations.Migration):

    dependencies = [
        ('ligoauth', '0004_update_gstlal_spiir_account'),
    ]

    operations = [
        migrations.RunPython(deactivate_old_and_add_new_accounts,
            activate_old_and_remove_new_accounts),
    ]
