# -*- coding: utf-8 -*-
# Generated by Django 1.11.20 on 2019-06-03 18:09
from __future__ import unicode_literals

from django.db import migrations

def populate_user_field(apps, schema_editor):
    X509Cert = apps.get_model('ligoauth', 'X509Cert')

    # Loop over all certificates, get first user, add as user ForeignKey
    for cert in X509Cert.objects.iterator():
        cert.user = cert.users.first()
        cert.save(update_fields=['user'])

        # Clear out users m2m field
        # NOTE: this isn't exactly "safe" if there is more than one
        # user associated with a certificate, but there *shouldn't* be
        # (no examples found in production DB after fixing some errors),
        # and even if there is, it should be cleared up after running
        # the 'update_user_accounts_from_ligo_ldap' script once or twice.
        cert.users.clear()


def populate_users_field(apps, schema_editor):
    X509Cert = apps.get_model('ligoauth', 'X509Cert')

    # Loop over all certificates and add user to users m2m field
    for cert in X509Cert.objects.iterator():
        cert.users.add(cert.user)

        # Clear out user ForeignKey
        cert.user = None
        cert.save(update_fields=['user'])


class Migration(migrations.Migration):

    dependencies = [
        ('ligoauth', '0041_x509cert_add_user_foreignkey'),
    ]

    operations = [
        migrations.RunPython(populate_user_field, populate_users_field),
    ]
