# -*- coding: utf-8 -*-
# Generated by Django 1.11.5 on 2017-10-25 17:25
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
from django.apps import apps as django_apps

def guardian_post_migrate_signal(apps, schema_editor):

    # We need to emit a post-migrate signal so that django.contrib.auth
    # will create the permission objects corresponding to the new
    # GroupObjectPermission and UserObjectPermission models that were
    # created by django-guardian. This is definitely not the nicest
    # way of doing things, but it's what we are stuck with for now.
    print("\n\nManually emitting post-migrate signal for guardian:")
    guardian_config = django_apps.get_app_config('guardian')
    models.signals.post_migrate.send(
        sender=guardian_config,
        app_config=guardian_config,
        verbosity=2,
        interactive=False,
        using=schema_editor.connection.alias,
    )

class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('auth', '0007_auto_20150708_1134'),
        ('contenttypes', '0002_remove_content_type_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='GroupObjectPermission',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('object_pk', models.CharField(max_length=255, verbose_name='object ID')),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType')),
                ('group', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='auth.Group')),
                ('permission', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='auth.Permission')),
            ],
        ),
        migrations.CreateModel(
            name='UserObjectPermission',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('object_pk', models.CharField(max_length=255, verbose_name='object ID')),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType')),
                ('permission', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='auth.Permission')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.AlterUniqueTogether(
            name='userobjectpermission',
            unique_together=set([('user', 'permission', 'object_pk')]),
        ),
        migrations.AlterUniqueTogether(
            name='groupobjectpermission',
            unique_together=set([('group', 'permission', 'object_pk')]),
        ),
        migrations.RunPython(guardian_post_migrate_signal, reverse_code=None),
    ]
